// Generated by CoffeeScript 1.6.3
/*
Copyright (c) 2012-2013 [DeftJS Framework Contributors](http://deftjs.org)
Open source under the [MIT License](http://en.wikipedia.org/wiki/MIT_License).
*/


/**
* Event bus for live component selectors.
* @private
*/


(function() {
  Ext.define('Deft.event.LiveEventBus', {
    alternateClassName: ['Deft.LiveEventBus'],
    requires: ['Ext.Component', 'Ext.ComponentManager', 'Deft.event.LiveEventListener'],
    singleton: true,
    constructor: function() {
      this.listeners = [];
    },
    destroy: function() {
      var listener, _i, _len, _ref;
      _ref = this.listeners;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        listener = _ref[_i];
        listener.destroy();
      }
      this.listeners = null;
    },
    addListener: function(container, selector, eventName, fn, scope, options) {
      var listener;
      listener = Ext.create('Deft.event.LiveEventListener', {
        container: container,
        selector: selector,
        eventName: eventName,
        fn: fn,
        scope: scope,
        options: options
      });
      this.listeners.push(listener);
    },
    removeListener: function(container, selector, eventName, fn, scope) {
      var listener;
      listener = this.findListener(container, selector, eventName, fn, scope);
      if (listener != null) {
        Ext.Array.remove(this.listeners, listener);
        listener.destroy();
      }
    },
    on: function(container, selector, eventName, fn, scope, options) {
      return this.addListener(container, selector, eventName, fn, scope, options);
    },
    un: function(container, selector, eventName, fn, scope) {
      return this.removeListener(container, selector, eventName, fn, scope);
    },
    findListener: function(container, selector, eventName, fn, scope) {
      var listener, _i, _len, _ref;
      _ref = this.listeners;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        listener = _ref[_i];
        if (listener.container === container && listener.selector === selector && listener.eventName === eventName && listener.fn === fn && listener.scope === scope) {
          return listener;
        }
      }
      return null;
    },
    register: function(component) {
      component.on('added', this.onComponentAdded, this);
      component.on('removed', this.onComponentRemoved, this);
    },
    unregister: function(component) {
      component.un('added', this.onComponentAdded, this);
      component.un('removed', this.onComponentRemoved, this);
    },
    onComponentAdded: function(component, container, eOpts) {
      var listener, _i, _len, _ref;
      _ref = this.listeners;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        listener = _ref[_i];
        listener.register(component);
      }
    },
    onComponentRemoved: function(component, container, eOpts) {
      var listener, _i, _len, _ref;
      _ref = this.listeners;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        listener = _ref[_i];
        listener.unregister(component);
      }
    }
  }, function() {
    Ext.Function.interceptAfter(Ext.ComponentManager, 'register', function(component) {
      if (component.isObservable && component.on && component.un) {
        Deft.event.LiveEventBus.register(component);
      }
    });
    Ext.Function.interceptAfter(Ext.ComponentManager, 'unregister', function(component) {
      if (component.isObservable && component.on && component.un) {
        Deft.event.LiveEventBus.unregister(component);
      }
    });
  });

}).call(this);
